#include <R.h>
#include <Rmath.h>
#include <Rinternals.h>

#ifdef _OPENMP
#include <omp.h>
#endif

extern "C" {

  SEXP cRMaternOMPSafe(SEXP D_r, SEXP nrow_r, SEXP ncol_r, SEXP phi_r, SEXP nu_r, SEXP nThreads_r){
    
    double *D = REAL(D_r);
    int nrow = INTEGER(nrow_r)[0];
    int ncol = INTEGER(ncol_r)[0];
    double phi = REAL(phi_r)[0];
    double nu = REAL(nu_r)[0];
    int nThreads = INTEGER(nThreads_r)[0];

#ifdef _OPENMP
    omp_set_num_threads(nThreads);
#else
    if(nThreads > 1){
      warning("n.omp.threads = %i requested however source code was not compiled with OpenMP support.", nThreads);
      nThreads = 1;
    }
#endif
    
    //other stuff
    int i, nProtect = 0;
    SEXP R_r;
    PROTECT(R_r = allocMatrix(REALSXP, nrow, ncol)); nProtect++;
    double *R = REAL(R_r);

    //thread safe stuff
    int threadID = 0;
    int bessel_ws_inc = static_cast<int>(1.0+nu);//see bessel_k.c for working space needs
    double *bessel_ws = (double *) R_alloc(nThreads*bessel_ws_inc, sizeof(double));
    
    //create the correlation matrix (now thread-safe)
#ifdef _OPENMP
#pragma omp parallel for private(threadID)
#endif
    for(i = 0; i < nrow*ncol; i++){
      
      if(D[i]*phi > 0.0){
	
#ifdef _OPENMP
	threadID = omp_get_thread_num();
#endif
	
	R[i] = pow(D[i]*phi, nu)/(pow(2, nu-1)*gammafn(nu))*bessel_k_ex(D[i]*phi, nu, 1.0, &bessel_ws[threadID*bessel_ws_inc]);
      }else{
	R[i] = 1.0;
      }
    }

    //unprotect
    UNPROTECT(nProtect);
    
    return(R_r);
  }  
}
