##Note, depending on your OS and threaded BLAS library dMvN's
##n.omp.thread might work at controlling the number of threads.
##If not, then running "export OMP_NUM_THREADS=n", where n is
##the number of desired threads, on the terminal prior to
##starting R or R CMD BATCH try-it-out.R should work.

rm(list=ls())

dyn.load("dMvN.so")
source("dMvN.R")

rmvn <- function(n, mu=0, V = matrix(1)){
  p <- length(mu)
  if(any(is.na(match(dim(V),p))))
    stop("Dimension problem!")
  D <- chol(V)
  t(matrix(rnorm(n*p), ncol=p)%*%D + rep(mu,rep(n,p)))
}

set.seed(1)

n <- 5000
coords <- cbind(runif(n,0,1), runif(n,0,1))
X <- as.matrix(cbind(1, rnorm(n)))

B <- as.matrix(c(1,5))
p <- length(B)

##Exponential spatial covariance with a nugget
sigma.sq <- 2
tau.sq <- 0.1
phi <- 3/0.1

D <- as.matrix(dist(coords))
C <- sigma.sq*exp(-phi*D)+diag(tau.sq,n)
mu <- X%*%B
y <- rmvn(1, mu, C)

##Evaluate log likelihood
out <- dMvN(y, mu, C, n.omp.thread=2)

out$log.density
out$sys.time

##Compare timing to:
library(mvtnorm)

start.time <- proc.time()
dmvnorm(y[,1], mu[,1], C, log=TRUE)
proc.time()-start.time
